# frozen_string_literal: true

require 'spec_helper'

describe GitlabSchema.types['Vulnerability'] do
  let_it_be(:project) { create(:project) }
  let_it_be(:user) { create(:user) }
  let_it_be(:vulnerability) { create(:vulnerability, project: project) }

  let(:fields) do
    %i[id title description state severity report_type vulnerability_path location]
  end

  before do
    stub_licensed_features(security_dashboard: true)

    project.add_developer(user)
  end

  subject { GitlabSchema.execute(query, context: { current_user: user }).as_json }

  it { expect(described_class).to have_graphql_fields(fields) }
  it { expect(described_class).to require_graphql_authorizations(:read_vulnerability) }

  describe 'vulnerability_path' do
    let(:query) do
      %(
        query {
          project(fullPath:"#{project.full_path}") {
            name
            vulnerabilities {
              nodes {
                vulnerabilityPath
              }
            }
          }
        }
      )
    end

    it "is the path to the vulnerability's detail page" do
      vulnerabilities = subject.dig('data', 'project', 'vulnerabilities', 'nodes')

      expect(vulnerabilities.first['vulnerabilityPath']).to eq(
        ::Gitlab::Routing.url_helpers.project_security_vulnerability_path(project, vulnerability)
      )
    end
  end

  describe 'location' do
    let_it_be(:location) do
      {
        'end_line' => 666,
        'file' => 'vulnerable-file.js',
        'start_line' => 420
      }
    end

    let(:query) do
      %(
        query {
          project(fullPath:"#{project.full_path}") {
            name
            vulnerabilities {
              nodes {
                location
              }
            }
          }
        }
      )
    end

    before do
      create(
        :vulnerabilities_occurrence,
        vulnerability: vulnerability,
        raw_metadata: {
          location: location
        }.to_json
      )
    end

    it "is the JSON metadata for the vulnerability's location" do
      vulnerabilities = subject.dig('data', 'project', 'vulnerabilities', 'nodes')

      expect(JSON.parse(vulnerabilities.first['location'])).to eq(location)
    end
  end
end
